@@ -60,10 +60,6 @@
 
 #include "timeval.h"
 
-#ifndef O_BINARY
-#  define O_BINARY 0
-#endif
-
 
 /**
  * Open a file for read, and also put its size into @p fsize.
@@ -82,7 +78,7 @@
  *
  * The caller is responsible for closing @p ifd.
  **/
-int dcc_open_read(const char *fname, int *ifd, off_t *fsize)
+static int dcc_open_read(const char *fname, int *ifd, off_t *fsize)
 {
     struct stat buf;
     
@@ -130,39 +126,6 @@ static void dcc_calc_rate(size_t size_out,
 }
 
 
-/**
- * Transmit a file and print timing statistics.  Only used for big files.
- *
- * Wrapper around dcc_x_file(). 
- **/
-int dcc_x_file_timed(int ofd, const char *fname, const char *token,
-                     size_t *size_out)
-{
-    struct timeval before, after;
-    int ret;
-    size_t size;
-
-    if (gettimeofday(&before, NULL))
-        rs_log_warning("gettimeofday failed");
-
-    ret = dcc_x_file(ofd, fname, token, &size);
-    if (size_out)
-        *size_out = size;           /* separate to allow for size_out NULL */
-
-    if (gettimeofday(&after, NULL)) {
-        rs_log_warning("gettimeofday failed");
-    } else {
-        double secs, rate;
-        
-        dcc_calc_rate(size, &before, &after, &secs, &rate);
-        rs_log_info("%ld bytes sent in %.6fs, rate %.0fkB/s",
-                    (long) size, secs, rate);
-    }
-
-    return ret;
-}
-
-
 /**
  * Transmit from a local file to the network.
  *
@@ -234,8 +197,8 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
      * might also help in the dangerous case of some other process still
      * reading from the file.
      *
-     * I guess checking for size 0 helps avoid removing /dev/null when running
-     * as root.
+     * Checking for size 0 means that we won't unlink special files like
+     * /dev/null or fifos.
      *
      * However, failure to remove the file does not cause a warning; we may
      * not have write permission on the directory.  The main case of this is
@@ -250,7 +213,9 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
             }
         }
     } else {
-        rs_trace("stat %s failed: %s", filename, strerror(errno));
+        if (errno != ENOENT) {
+            rs_trace("stat %s failed: %s", filename, strerror(errno));
+        }
         /* continue */
     }
 
@@ -264,7 +229,7 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
     close_ret = dcc_close(ofd);
 
     if (!ret && !close_ret) {
-        rs_log_info("received %d bytes to file %s", len, filename);
+        rs_trace("received %d bytes to file %s", len, filename);
         return 0;
     }
 
