@@ -52,6 +52,7 @@
 #include "exitcode.h"
 #include "clinet.h"
 #include "util.h"
+#include "netutil.h"
 
 
 /**
@@ -61,22 +62,74 @@
  *
  * @todo In error messages, show the name of the relevant host.
  * Should do this even in readx(), etc.
- *
- * @todo Connect can take a very long time to fail.  We probably need a
- * userspace timeout on it, e.g. by doing a nonblocking connect and select()
- * on it.  Check how to do this properly.
  **/
 
 
+/* TODO: Split out select timeout into a function that can be applied to any
+ * fd.  Then do it on SSH connections too.
+ */
+
+/* TODO: After connecting, perhaps try to read 0 bytes to see if there's an
+ * error.  */
+
+
+/**
+ * Connect a socket to a server, with a timeout.
+ *
+ * Basic mechanism is: make the socket nonblocking, start it connecting, and
+ * then keep selecting on it until we either lose, win, or get bored.
+ **/
+static int dcc_connect_timed(int fd,
+                             struct sockaddr *sa,
+                             int salen)
+{
+    char *s;
+    int rs;
+    int ret;
+
+    dcc_sockaddr_to_string(sa, salen, &s);
+
+    dcc_set_nonblocking(fd); 
+
+    rs_trace("nonblocking connect to %s", s);
+
+    /* kick off connection attempt */
+    rs = connect(fd, sa, salen);
+    if (rs == -1 && errno != EINPROGRESS ) {
+        rs_log_error("failed to connect to %s: %s",
+                     s, strerror(errno));
+        goto out_failed;
+    }
+    
+    if ((ret = dcc_select_for_read(fd)))
+        goto out_failed;
+
+    /* If the connection failed we may not yet have noticed the error; we'd
+     * need to check the SOL_SOCKET,SO_ERROR option.  For our current purposes
+     * it is fine to just assume it worked and trap the error when we try to
+     * write.
+     *
+     * See <http://cr.yp.to/docs/connect.html> for a good discussion. */
+    
+    dcc_set_blocking(fd); 
+    free(s);
+    return 0;
+    
+    out_failed:
+    free(s);
+    return EXIT_CONNECT_FAILED;
+}
+
 
 /**
  * @p host and @p port are only here to aid printing debug messages.
  **/
 static int dcc_connect_by_addr(const char *host, int port,
-                               struct sockaddr *sa, size_t sa_len,
+                               struct sockaddr *sa, size_t salen,
                                int *p_fd)
 {
     int fd;
+    int ret;
 
     fd = socket(sa->sa_family, SOCK_STREAM, 0);
     
@@ -85,16 +138,17 @@ static int dcc_connect_by_addr(const char *host, int port,
 	return EXIT_CONNECT_FAILED;
     }
 
-    if (connect(fd, sa, sa_len)) {
-        /* TODO: Print raw sockaddr form in some readable way. */
-        rs_log_error("failed to connect to %s port %d: %s", host, port, 
-                     strerror(errno));
+    if ((ret = dcc_connect_timed(fd, sa, salen))) {
         close(fd);
-	return EXIT_CONNECT_FAILED;
+	return ret;
     }
 
-        /* TODO: Print raw sockaddr form in some readable way. */
-    rs_trace("client got connection to %s port %d on fd%d", host, port, fd);
+    if (rs_trace_enabled()) {
+        char *s;
+        dcc_sockaddr_to_string(sa, salen, &s);
+        rs_trace("client got connection to %s port %d on fd%d", host, port, fd);
+        free(s);
+    }
 
     *p_fd = fd;
     return 0;
