@@ -1,4 +1,4 @@
-/* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78 -*-
+/* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
  *
@@ -43,7 +43,8 @@
 #include "bulk.h"
 #include "clirpc.h"
 #include "hosts.h"
-
+#include "state.h"
+#include "state.h"
 
 /**
  * @file
@@ -76,12 +77,7 @@ static int dcc_x_argv(int fd, char **argv)
         return EXIT_PROTOCOL_ERROR;
     
     for (i = 0; i < argc; i++) {
-        size_t len;
-
-        len = strlen(argv[i]);
-        if ((ret = dcc_x_token_int(fd, "ARGV", (unsigned) len)))
-            return ret;
-        if ((ret = dcc_writex(fd, argv[i], len)))
+        if ((ret = dcc_x_token_string(fd, "ARGV", argv[i])))
             return ret;
     }
 
@@ -92,10 +88,9 @@ static int dcc_x_argv(int fd, char **argv)
 static int dcc_send_job(int net_fd,
                         char **argv,
                         pid_t cpp_pid,
-                        int *status,
+                        unsigned *status,
                         const char *cpp_fname)
 {
-    long stime_usec, utime_usec;
     int ret;
 
     if ((ret = dcc_x_req_header(net_fd))
@@ -103,11 +98,11 @@ static int dcc_send_job(int net_fd,
         return ret;
 
     if (cpp_pid) {
+        dcc_note_state(STATE_CPP, NULL, NULL);
         /* Wait for cpp to finish (if not already done), check the
          * result, then send the .i file */
         
-        if ((ret = dcc_collect_child(cpp_pid, status, &utime_usec, &stime_usec))
-            || (ret = dcc_report_rusage("cpp", utime_usec, stime_usec)))
+        if ((ret = dcc_collect_child("cpp", cpp_pid, status)))
             return ret;
 
         /* Although cpp failed, there is no need to try running the command
@@ -126,7 +121,8 @@ static int dcc_send_job(int net_fd,
         return EXIT_IO_ERROR;
     }
 
-    if ((ret = dcc_x_file_timed(net_fd, cpp_fname, "DOTI", NULL)))
+    dcc_note_state(STATE_SEND, NULL, NULL);
+    if ((ret = dcc_x_file(net_fd, cpp_fname, "DOTI", NULL)))
         return ret;
 
     rs_trace("client finished sending request to server");
@@ -151,7 +147,7 @@ static int dcc_send_job(int net_fd,
 int dcc_send_job_corked(int net_fd,
 			char **argv,
 			pid_t cpp_pid,
-			int *status,
+                        unsigned *status,
 			const char *cpp_fname)
 {
     int ret;
@@ -171,7 +167,7 @@ int dcc_send_job_corked(int net_fd,
  **/
 int dcc_r_result_header(int ifd)
 {
-    int vers;
+    unsigned vers;
     int ret;
     
     if ((ret = dcc_r_token_int(ifd, "DONE", &vers)))
@@ -189,20 +185,26 @@ int dcc_r_result_header(int ifd)
 }
 
 
-int dcc_r_cc_status(int ifd, int *status)
+int dcc_r_cc_status(int ifd, unsigned *status)
 {
     return dcc_r_token_int(ifd, "STAT", status);
 }
 
 
-int dcc_retrieve_results(int net_fd, int *status, const char *output_fname)
+int dcc_retrieve_results(int net_fd, unsigned *status, const char *output_fname)
 {
-    int len;
+    unsigned len;
     int ret;
-    int o_len;
+    unsigned o_len;
 
-    if ((ret = dcc_r_result_header(net_fd))
-        || (ret = dcc_r_cc_status(net_fd, status))
+    if ((ret = dcc_r_result_header(net_fd)))
+        return ret;
+
+    /* We've started to see the response, so the server is done
+     * compiling. */
+    dcc_note_state(STATE_RECEIVE, NULL, NULL);
+
+    if ((ret = dcc_r_cc_status(net_fd, status))
         || (ret = dcc_r_token_int(net_fd, "SERR", &len))
         || (ret = dcc_r_file_body(STDERR_FILENO, net_fd, len))
         || (ret = dcc_r_token_int(net_fd, "SOUT", &len))
@@ -213,7 +215,7 @@ int dcc_retrieve_results(int net_fd, int *status, const char *output_fname)
     /* Previously we would skip retrieving the .o file unless the compiler
      * completed successfully, but it seems cleaner to have the protocol
      * always the same and to always drain the network.  The server always
-     * sends an 0 byte file on failure anyhow. */
+     * sends DOTO 0 on failure anyhow. */
 
     if (o_len)
         return dcc_r_file_timed(net_fd, output_fname, o_len);
