@@ -51,6 +51,22 @@
 #include <errno.h>
 #include <syslog.h>
 
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <sys/param.h>
+#include <sys/socket.h>
+
+#include <netinet/in.h>
+#include <netinet/tcp.h>
+
+#ifdef HAVE_ARPA_NAMESER_H
+#  include <arpa/nameser.h>
+#endif
+
+#include <arpa/inet.h>
+
+
 #include "exitcode.h"
 #include "distcc.h"
 #include "trace.h"
@@ -59,6 +75,9 @@
 #include "dopt.h"
 #include "setuid.h"
 #include "srvnet.h"
+#include "daemon.h"
+#include "tempfile.h"
+#include "types.h"
 
 
 /* for trace.c */
@@ -134,19 +153,27 @@ int main(int argc, char *argv[])
     /* Discard privileges before opening log so that if it's created, it has
      * the right ownership. */
     dcc_setup_real_log();
+
+    /* Do everything from temporary directory.  Makes things more
+     * reproducible; avoids tempdir itself being deleted; allows start
+     * directory to be unmounted, ... */
+    if ((ret = dcc_enter_tempdir()))
+        goto out;
      
-    if ((requested_path = getenv("DISTCCD_PATH")) != NULL)
-        ret = dcc_set_path(requested_path);
-    else
-        ret = dcc_trim_path(NULL);
-    if (ret)
-        dcc_exit(ret);
+    if ((requested_path = getenv("DISTCCD_PATH")) != NULL) {
+        if ((ret = dcc_set_path(requested_path)))
+            goto out;
+    } else {
+        if ((ret = dcc_trim_path(NULL)))
+            goto out;
+    }
 
     if (dcc_should_be_inetd())
         ret = dcc_inetd_server();
     else
         ret = dcc_standalone_server();
     
+    out:
     dcc_exit(ret);
 }
 
@@ -196,6 +223,7 @@ static void dcc_setup_real_log(void)
     }
     
     rs_remove_all_loggers();
+    openlog("distccd", LOG_PID, LOG_DAEMON);
     rs_add_logger(rs_logger_syslog, RS_LOG_DEBUG, NULL, 0);
 }
 
@@ -206,11 +234,21 @@ static void dcc_setup_real_log(void)
 static int dcc_inetd_server(void)
 {
     int ret, close_ret;
+    struct sockaddr_storage ss;
+    struct sockaddr *psa = (struct sockaddr *) &ss;
+    socklen_t len = sizeof ss;
     
     rs_log_info("inetd server started (version %s, built %s %s)", PACKAGE_VERSION,
                 __DATE__, __TIME__);
-    
-    ret = dcc_accept_job(STDIN_FILENO);
+
+    /* XXX: ssh seems to open us using a unix domain socket.  Is that always
+     * the case? */
+    if ((getpeername(STDIN_FILENO, psa, &len) == -1)) {
+        rs_log_warning("failed to get peer name: %s", strerror(errno));
+        len = 0;
+    }
+
+    ret = dcc_service_job(STDIN_FILENO, psa, len);
 
     close_ret = dcc_close(STDIN_FILENO);
 
