@@ -82,45 +82,26 @@ int dcc_argv_len(char **a);
 /* A macro so that we get the right __FUNCTION__ in the trace message.
  *
  * We condition on rs_trace_enabled so that we don't do the to-string
- * conversion unless the user will actually see the result.  */
+ * conversion unless the user will actually see the result, because it's a
+ * little expensive.  */
 #define dcc_trace_argv(_message, _argv)         \
     if (rs_trace_enabled()) {                   \
         char *_astr;                            \
         _astr = dcc_argv_tostr(_argv);          \
         rs_trace("%s: %s", _message, _astr);    \
         free(_astr);                            \
-    } else while (0)
+    } else {}
 
 
 /* dopt.c */
 extern struct dcc_allow_list *opt_allowed;
 int distccd_parse_options(int argc, const char *argv[]);
 
-/* serve.c */
-int dcc_accept_job(int fd);
-
 /* help.c */
 int dcc_trace_version(void);
 int dcc_show_copyright(void);
 int dcc_show_version(const char *prog);
 
-/* daemon.c */
-void dcc_server_child(int) NORETURN;
-int dcc_refuse_root(void);
-int dcc_set_lifetime(void);
-
-/* dsignal.c */
-void dcc_catch_signals(void);
-void dcc_ignore_sighup(void);
-
-/* dparent.c */
-int dcc_standalone_server(void);
-void dcc_remove_pid(void);
-
-
-/* bulk.c */
-int dcc_open_read(const char *fname, int *ifd, off_t *fsize);
-
 
 /* hosts.c */
 int dcc_parse_hosts_env(struct dcc_hostdef **ret_list,
@@ -146,5 +127,25 @@ int dcc_recursion_safeguard(void);
 int dcc_support_masquerade(char *argv[], char *progname, int *);
 
 
+
 #define DISTCC_DEFAULT_PORT 3632
 
+
+
+#ifndef WAIT_ANY
+#  define WAIT_ANY (-1)
+#endif
+
+
+struct sockaddr;
+
+#ifndef HAVE_SOCKADDR_STORAGE
+#  define sockaddr_storage sockaddr_in
+#endif
+
+#ifndef O_BINARY
+#  define O_BINARY 0
+#endif
+
+
+void dcc_set_trace_from_env(void);
