@@ -21,9 +21,10 @@
  */
 
 extern int arg_port;
-extern int arg_nchildren;
+extern int arg_max_jobs;
 extern const char *arg_pid_file;
 extern int opt_no_fork;
+extern int opt_no_prefork;
 extern int opt_no_detach;
 extern int opt_daemon_mode, opt_inetd_mode;
 extern const char *arg_log_file;
