@@ -20,11 +20,10 @@
  * USA
  */
 
-
-		/* I don't know that atheists should be considered as
-		 * citizens, nor should they be considered patriots.
-		 * This is one nation under God.
-		 * -- Vice President, George H. W. Bush, 1987 */
+                    /* Near is thy forgetfulness of all things; and near the
+                     * forgetfulness of thee by all.
+                     *                 -- Marcus Aurelius
+                     */
 
 
 /**
@@ -71,18 +70,22 @@
 #include "tempfile.h"
 #include "srvnet.h"
 #include "types.h"
+#include "daemon.h"
 
-#ifndef WAIT_ANY
-#  define WAIT_ANY (-1)
-#endif
-
-static int dcc_start_child(int listen_fd, int);
-static void dcc_parent_loop(int listen_fd) NORETURN;
-static int dcc_serve_connection(int, int acc_fd);
+static int dcc_start_server_child(int listen_fd, int accept_fd, struct sockaddr *,
+                                  int);
+static void dcc_nofork_parent(int listen_fd) NORETURN;
+static void dcc_forking_parent(int listen_fd) NORETURN;
 static void dcc_detach(void);
 static void dcc_save_pid(pid_t);
+int dcc_nkids = 0;
 
-static int nkids = 0;
+
+/**
+ * In forking or prefork mode, the maximum number of connections we want to
+ * allow at any time.
+ **/
+int dcc_max_kids = 0;
 
 
 /**
@@ -96,19 +99,27 @@ int dcc_standalone_server(void)
     int n_cpus;
     int ret;
 
+    rs_log(RS_LOG_INFO|RS_LOG_NONAME,
+           "distccd (version %s, built %s %s)",
+           PACKAGE_VERSION, __DATE__, __TIME__);
+
     if ((ret = dcc_socket_listen(arg_port, &listen_fd, opt_listen_addr)) != 0)
         return ret;
 
     set_cloexec_flag(listen_fd, 1);
 
-    rs_log(RS_LOG_INFO|RS_LOG_NONAME,
-           "distccd (version %s, built %s %s) listening on port %d",
-           PACKAGE_VERSION, __DATE__, __TIME__, arg_port);
-
     if (dcc_ncpus(&n_cpus) == 0) 
-        rs_log_info("%d CPU%s online", n_cpus, n_cpus == 1 ? "" : "s");
+        rs_log_info("%d CPU%s online on this server", n_cpus, n_cpus == 1 ? "" : "s");
 
-    dcc_catch_signals();
+    /* By default, allow up to 3 children per CPU.  If they're all busy at the
+     * same time that shouldn't be too bad.  It's likely that some of them
+     * will be idle some of the time while they're reading or writing. */
+    if (arg_max_jobs)
+        dcc_max_kids = arg_max_jobs;
+    else
+        dcc_max_kids = 3 * n_cpus;
+    
+    rs_log_info("allowing up to %d active jobs", dcc_max_kids);
 
     if (!opt_no_detach) {
         /* Don't go into the background until we're listening and
@@ -116,10 +127,47 @@ int dcc_standalone_server(void)
          * detaches, we know we can go ahead and try to connect.  */
         dcc_detach();
     } else {
+        /* Still create a new process group, even if not detached */
+        rs_trace("not detaching");
+        if ((ret = dcc_new_pgrp()) != 0)
+            return ret;
         dcc_save_pid(getpid());
     }
 
-    dcc_parent_loop(listen_fd);		/* noreturn */
+    /* Don't catch signals until we've detached or created a process group. */
+    dcc_daemon_catch_signals();
+
+    /* This is called in the master daemon, whether that is detached or
+     * not.  */
+    dcc_master_pid = getpid();
+
+    if (opt_no_fork)
+        dcc_nofork_parent(listen_fd);
+    else if (opt_no_prefork) 
+        dcc_forking_parent(listen_fd);		/* noreturn */
+    else
+        return dcc_preforking_parent(listen_fd);
+}
+
+
+
+static void dcc_log_child_exited(pid_t kid,
+                                 int status)
+{
+    if (WIFSIGNALED(status)) {
+        int sig = WTERMSIG(status);
+        int severity = sig == SIGTERM ? RS_LOG_INFO : RS_LOG_ERR;
+        
+#ifdef HAVE_STRSIGNAL
+        /* take off every sig! for great justice. */
+        rs_log(severity, "child %d: %s", (int) kid, strsignal(sig));
+#else
+        rs_log_error("child %d: signal %d", (int) kid, sig);
+#endif
+    } else if (WIFEXITED(status)) {
+        rs_log_info("child %d exited: exit status %d",
+                    (int) kid, WEXITSTATUS(status));
+    }
 }
 
 
@@ -127,28 +175,30 @@ int dcc_standalone_server(void)
 /**
  * @sa dcc_wait_child(), which is used by a process that wants to do a blocking
  * wait for some task like cpp or gcc.
+ *
+ * @param must_reap If True, don't return until at least one child has been
+ * collected.  Used when e.g. all our process slots are full.  In either case
+ * we keep going until all outstanding zombies are collected.
+ *
+ * FIXME: Are blocking waits meant to collect all of them, or just one?  At
+ * the moment it waits until all children have exited.
  **/
-static void dcc_reap_kids(void)
+void dcc_reap_kids(int must_reap)
 {
     while (1) {
         int status;
         pid_t kid;
 
-        kid = waitpid(WAIT_ANY, &status, WNOHANG);
+        kid = waitpid(WAIT_ANY, &status, must_reap ? 0 : WNOHANG);
         if (kid == 0) {
+            /* nobody has exited */
             break;
         } else if (kid != -1) {
             /* child exited */
-            --nkids;
-            rs_log_info("down to %d children", nkids);
-            
-            if (WIFSIGNALED(status)) {
-                rs_log_error("child %d exited on signal %d",
-                             (int) kid, WTERMSIG(status));
-            } else if (WIFEXITED(status)) {
-                rs_log_notice("child %d exited: exit status %d",
-                              (int) kid, WEXITSTATUS(status));
-            }
+            --dcc_nkids;
+            rs_trace("down to %d children", dcc_nkids);
+
+            dcc_log_child_exited(kid, status);
         } else if (errno == ECHILD) {
             /* No children left?  That's ok, we'll go back to waiting
              * for new connections. */
@@ -161,18 +211,24 @@ static void dcc_reap_kids(void)
             continue;       /* loop again */
         } else {
             rs_log_error("wait failed: %s", strerror(errno));
+            /* e.g. too many open files; nothing we can do */
             dcc_exit(EXIT_DISTCC_FAILED);
         }
+
+        /* If there are more children keep looking, but don't block once we've
+         * collected at least one. */
+        must_reap = FALSE;
     }
 }
 
 
 /**
- * Main loop for the parent process.  Basically it does nothing but
- * wait around to be signalled, or for its children to die.  What a
- * life!
+ * Main loop for no-fork mode.
+ *
+ * Much slower and may leak.  Should only be used when you want to run gdb on
+ * distccd.
  **/
-static void dcc_parent_loop(int listen_fd)
+static void dcc_nofork_parent(int listen_fd)
 {
     while (1) {
         int acc_fd;
@@ -181,6 +237,42 @@ static void dcc_parent_loop(int listen_fd)
 
         rs_log_info("waiting to accept connection");
 
+        cli_len = sizeof cli_addr;
+        acc_fd = accept(listen_fd, &cli_addr, &cli_len);
+        if (acc_fd == -1 && errno == EINTR) {
+            ; 
+        }  else if (acc_fd == -1) {
+            rs_log_error("accept failed: %s", strerror(errno));
+            dcc_exit(EXIT_CONNECT_FAILED);
+        } else {
+            dcc_service_job(acc_fd, &cli_addr, cli_len);
+            dcc_cleanup_tempfiles();
+        }
+    }
+}
+
+
+/**
+ * Main loop for the parent process, with the old behaviour of forking as
+ * connections are accepted.  Basically it does nothing but wait around to be
+ * signalled, or for its children to die.  What a life!
+ **/
+static void dcc_forking_parent(int listen_fd)
+{
+    while (1) {
+        struct sockaddr cli_addr;
+        socklen_t cli_len;
+        int acc_fd;
+
+        dcc_reap_kids(FALSE);
+
+        if (dcc_nkids >= dcc_max_kids) {
+            rs_trace("too busy for new connections, waiting for child to complete");
+            dcc_reap_kids(TRUE);
+        }
+
+        rs_log_info("waiting to accept connection");
+
         cli_len = sizeof cli_addr;
         acc_fd = accept(listen_fd, &cli_addr, &cli_len);
         if (acc_fd == -1 && errno == EINTR) {
@@ -189,67 +281,51 @@ static void dcc_parent_loop(int listen_fd)
             rs_log_error("accept failed: %s", strerror(errno));
             dcc_exit(EXIT_CONNECT_FAILED);
         } else {
-            dcc_serve_connection(listen_fd, acc_fd);
-        }
-
-        if (!opt_no_fork) {
-            /* Don't call if in no-fork mode, because that would
-             * incorrectly pick up compilers etc. */
-            dcc_reap_kids();
+            dcc_start_server_child(listen_fd, acc_fd, &cli_addr, cli_len);
         }
     }
 }
 
-/**
- * Called in the daemon parent when a connection is received.  Either
- * forks a new child, or handles it in the process (debug mode), as
- * appropriate.
- **/
-static int dcc_serve_connection(int listen_fd, int acc_fd)
-{
-    int ret = 0;
-    
-    if (opt_no_fork) {
-        ret = dcc_accept_job(acc_fd);
-        dcc_cleanup_tempfiles();
-    } else {
-        dcc_start_child(listen_fd, acc_fd);
-        ++nkids;
-        rs_log_info("up to %d children", nkids);
-    }
-    if (close(acc_fd)) {          /* in parent */
-        rs_log_error("failed to close accepted fd%d: %s", acc_fd,
-                     strerror(errno));
-    }
 
-    return ret;
-}
+
 
 /**
- * Fork a child to handle an incoming job.  Not called when in
- * opt_no_fork mode.
+ * Fork a child to handle an already-accepted incoming connecting.  Not called
+ * when in opt_no_fork mode.
  **/
-static int dcc_start_child(int listen_fd, int accepted_fd)
+static int dcc_start_server_child(int listen_fd, int accepted_fd,
+                                  struct sockaddr *cli_addr,
+                                  int cli_len)
 {
     pid_t kid;
     int ret, close_ret;
     
     kid = fork();
     if (kid == 0) {
-        close(listen_fd);
-        ret = dcc_accept_job(accepted_fd);
+        dcc_close(listen_fd);
+        ret = dcc_service_job(accepted_fd, cli_addr, cli_len);
         close_ret = dcc_close(accepted_fd);
         dcc_exit(ret ? ret : close_ret);
         /* doesn't return */
     } else if (kid == -1) {
         rs_log_error("fork failed: %s", strerror(errno));
         return EXIT_OUT_OF_MEMORY; /* probably */
+    } else {
+        /* in parent */
+        ++dcc_nkids;
+        rs_log_info("up to %d children", dcc_nkids);
+
+        if (close(accepted_fd)) {          /* in parent */
+            rs_log_error("failed to close accepted fd%d in parent: %s",
+                         accepted_fd, strerror(errno));
+        }
     }
 
     return 0;
 }
 
 
+
 /**
  * Save the pid of the child process into the pid file, if any.
  *
@@ -312,6 +388,7 @@ static void dcc_detach(void)
 {
     int i;
     pid_t pid;
+    pid_t sid;
     
     dcc_ignore_sighup();
 
@@ -330,8 +407,12 @@ static void dcc_detach(void)
 
     /* detach from the terminal */
 #ifdef HAVE_SETSID
-        setsid();
-#else
+    if ((sid = setsid()) == -1) {
+        rs_log_error("setsid failed: %s", strerror(errno));
+    } else {
+        rs_trace("setsid to session %d", (int) sid);
+    }
+#else /* no HAVE_SETSID */
 #ifdef TIOCNOTTY
     i = open("/dev/tty", O_RDWR);
     if (i >= 0) {
@@ -339,7 +420,8 @@ static void dcc_detach(void)
         close(i);
     }
 #endif /* TIOCNOTTY */
-#endif
+#endif /* not HAVE_SETSID */
+    
     /* make sure that stdin, stdout an stderr don't stuff things
        up (library functions, for example) */
     for (i=0;i<3;i++) {
