@@ -32,18 +32,16 @@
  *
  * Run compilers or preprocessors.
  *
- * Subtasks are always run in their own process groups.  (In this respect we
- * are a little like a job-control shell...)  This allows us to cleanly kill
- * the whole compiler if something goes wrong.
+ * The whole server is run in a separate process group and normally in a
+ * separate session.  (It is not a separate session in --no-detach debug
+ * mode.)  This allows us to cleanly kill off all children and all compilers
+ * when the parent is terminated.
  *
  * @todo On Cygwin, fork() must be emulated and therefore will be
  * slow.  It would be faster to just use their spawn() call, rather
  * than fork/exec.
  **/
 
-
-
-
 #include "config.h"
 
 #include <stdio.h>
@@ -120,17 +118,6 @@ int dcc_redirect_fds(const char *stdin_file,
 }
 
 
-int dcc_remove_if_exists(const char *fname)
-{
-    if (unlink(fname) && errno != ENOENT) {
-        rs_log_warning("failed to unlink %s: %s", fname,
-                       strerror(errno));
-        return EXIT_IO_ERROR;
-    }
-    return 0;
-}
-
-
 /**
  * Replace this program with another in the same process.
  *
@@ -139,7 +126,23 @@ int dcc_remove_if_exists(const char *fname)
  **/
 void dcc_execvp(char **argv)
 {
+    char *slash;
+    
     execvp(argv[0], argv);
+
+    /* If we're still running, the program was not found on the path.  One
+     * thing that might have happened here is that the client sent an absolute
+     * compiler path, but the compiler's located somewhere else on the server.
+     * In the absence of anything better to do, we search the path for its
+     * basename.
+     *
+     * Actually this code is called on both the client and server, which might
+     * cause unintnded behaviour in contrived cases, like giving a full path
+     * to a file that doesn't exist.  I don't think that's a problem. */
+
+    slash = strrchr(argv[0], '/');
+    if (slash)
+        execvp(slash + 1, argv);
     
     /* shouldn't be reached */
     rs_log_error("failed to exec %s: %s", argv[0], strerror(errno));
@@ -189,15 +192,23 @@ static int dcc_inside_child(char **argv,
 }
 
 
-void dcc_setpgid(pid_t pid, pid_t pgid)
+int dcc_new_pgrp(void)
 {
-    if (setpgid(pid, pgid) == 0)
-        rs_trace("setpgid(%ld, %ld)", (long) pid, (long) pgid);
-    else
-        rs_trace("setpgid(%ld, %ld) failed: %s",
-                 (long) pid, (long) pgid, strerror(errno));
+    /* If we're a session group leader, then we are not able to call
+     * setpgid().  However, setsid will implicitly have put us into a new
+     * process group, so we don't have to do anything. */
+    if (getpgid(0) == getpid()) {
+        rs_trace("already a process group leader");
+        return 0;
+    }
     
-    /* Continue anyhow i guess.  Failure seems unlikely.  */
+    if (setpgid(0, 0) == 0) {
+        rs_trace("entered process group");
+        return 0;
+    } else {
+        rs_trace("setpgid(0, 0) failed: %s", strerror(errno));
+        return EXIT_DISTCC_FAILED;
+    }
 }
 
 
@@ -272,7 +283,8 @@ static int sys_wait4(pid_t pid, int *status, int options, struct rusage *rusage)
  * parent only waits when it thinks a child has exited; the child
  * waits all the time.
  **/
-int dcc_collect_child(pid_t pid, int *wait_status, long *u_us, long *s_us)
+int dcc_collect_child(const char *what, pid_t pid,
+                      unsigned *wait_status)
 {
     struct rusage ru;
     pid_t ret_pid;
@@ -281,10 +293,16 @@ int dcc_collect_child(pid_t pid, int *wait_status, long *u_us, long *s_us)
         if ((ret_pid = sys_wait4(pid, wait_status, 0, &ru)) != -1) {
             /* This is not the main user-visible message, that comes from
              * critique_status(). */
-            rs_trace("child %ld terminated with status %#x",
-                     (long) ret_pid, *wait_status);
-            *u_us = ru.ru_utime.tv_sec * 1000000 + ru.ru_utime.tv_usec;
-            *s_us = ru.ru_stime.tv_sec * 1000000 + ru.ru_stime.tv_usec;
+            rs_trace("%s child %ld terminated with status %#x",
+                     what, (long) ret_pid, *wait_status);
+
+            rs_log_info("%s times: user %ld.%06lds, system %ld.%06lds, "
+                        "%ld minflt, %ld majflt",
+                        what,
+                        ru.ru_utime.tv_sec, ru.ru_utime.tv_usec,
+                        ru.ru_stime.tv_sec, ru.ru_stime.tv_usec,
+                        ru.ru_minflt, ru.ru_majflt);
+
             return 0;
         } else if (errno == EINTR) {
             rs_trace("wait4 was interrupted; retrying");
@@ -297,19 +315,6 @@ int dcc_collect_child(pid_t pid, int *wait_status, long *u_us, long *s_us)
 }
 
 
-/**
- * Report child's resource usage in readable format.
- **/
-int dcc_report_rusage(const char *command,
-                      long utime_usec,
-                      long stime_usec)
-{
-    rs_log_info("%s resource usage: %ld.%06lds user, %ld.%06lds system",
-                command, utime_usec/1000000, utime_usec%1000000,
-                stime_usec/1000000, stime_usec%1000000);
-    return 0;
-}
-
 
 /**
  * Analyze and report on a command's exit code.
@@ -320,7 +325,8 @@ int dcc_report_rusage(const char *command,
  * @returns 0 if the command succeeded; 128+SIGNAL if it stopped on a
  * signal; otherwise the command's exit code.
  **/
-int dcc_critique_status(int s, const char *command,
+int dcc_critique_status(unsigned s,
+                        const char *command,
                         struct dcc_hostdef *host,
                         int verbose)
 {
@@ -331,14 +337,22 @@ int dcc_critique_status(int s, const char *command,
     if (verbose)
         logmode = RS_LOG_ERR | RS_LOG_NONAME;
     else
-        logmode = RS_LOG_INFO;
+        logmode = RS_LOG_INFO | RS_LOG_NONAME;
     
     if (WIFSIGNALED(s)) {
+#ifdef HAVE_STRSIGNAL
         rs_log(logmode,
-               "%s on %s died with signal %d%s",
+               "%s on %s:%s %s",
+               command, host->hostdef_string,
+               strsignal(WTERMSIG(s)),
+               WCOREDUMP(s) ? " (core dumped)" : "");
+#else
+        rs_log(logmode,
+               "%s on %s terminated by signal %d%s",
                command, host->hostdef_string,
                WTERMSIG(s),
                WCOREDUMP(s) ? " (core dumped)" : "");
+#endif
         /* Unix convention is to return 128+signal when a subprocess crashes. */
         return 128 + WTERMSIG(s);
     } else if (WEXITSTATUS(s)) {
@@ -351,7 +365,7 @@ int dcc_critique_status(int s, const char *command,
                command, host->hostdef_string, WEXITSTATUS(s));
         return WEXITSTATUS(s);
     } else {
-        rs_log(RS_LOG_INFO,
+        rs_log(RS_LOG_INFO|RS_LOG_NONAME,
                "%s on %s completed ok", command, host->hostdef_string);
         return 0;
     }
