@@ -29,15 +29,13 @@ void dcc_execvp(char **argv) NORETURN;
 int dcc_spawn_child(char **argv, pid_t *pidptr,
                     const char *, const char *, const char *);
 
-int dcc_collect_child(pid_t pid, int *wait_status, long *, long *);
-int dcc_critique_status(int s, const char *, struct dcc_hostdef *host,
+int dcc_collect_child(const char *what, pid_t pid,
+                      unsigned *wait_status);
+int dcc_critique_status(unsigned s, const char *, struct dcc_hostdef *host,
                         int verbose);
 void dcc_note_execution(struct dcc_hostdef *host, char **argv);
-int dcc_report_rusage(const char *command,
-                      long utime_usec,
-                      long stime_usec);
 
-void dcc_setpgid(pid_t pid, pid_t pgid);
+int dcc_new_pgrp(void);
 void dcc_reset_signal(int whichsig);
 
 #ifndef W_EXITCODE
