@@ -35,9 +35,10 @@
  * The grammar of this variable is, informally:
  *
  * DISTCC_HOSTS = HOST ...
- * HOSTID = SSH_HOST | TCP_HOST
+ * HOSTID = SSH_HOST | TCP_HOST | OLDSTYLE_TCP_HOST
  * SSH_HOST = USER@HOSTID[/MUL][:COMMAND]
- * TCP_HOST = HOSTID[/MUL][:PORT]
+ * TCP_HOST = HOSTID[:PORT][/MUL]
+ * OLDSTYLE_TCP_HOST = HOSTID[/MUL][:PORT]
  * HOSTID = HOSTNAME | IPV4 | '['IPV6']'
  *
  * Any amount of whitespace may be present between hosts.
@@ -91,6 +92,7 @@
 #include "util.h"
 #include "hosts.h"
 #include "exitcode.h"
+#include "snprintf.h"
 
 const int dcc_default_port = DISTCC_DEFAULT_PORT;
 
@@ -165,22 +167,30 @@ int dcc_get_hostlist(struct dcc_hostdef **ret_list,
 }
 
 
-
+/**
+ * Parse an optionally present multiplier.
+ *
+ * *psrc is the current parse cursor; it is advanced over what is read.
+ *
+ * If a multiplier is present, *psrc points to a substring starting with '/'.
+ * The host defintion is updated to the numeric value following.  Otherwise
+ * the hostdef is unchanged.
+ **/
 static int dcc_parse_multiplier(const char **psrc, struct dcc_hostdef *hostdef)
 {
-    char *mul;
     const char *token = *psrc;
-    int ret;
 
     if ((*psrc)[0] == '/') {
+        int val;
         (*psrc)++;
-        if ((ret = dcc_dup_part(psrc, &mul, "/: \t\n\f")) != 0)
-            return ret;
-        if (!mul || atoi(mul) == 0) {
+        val = atoi(*psrc);
+        if (val == 0) {
             rs_log_error("bad multiplier \"%s\" in host specification", token);
             return EXIT_BAD_HOSTSPEC;
         }
-        hostdef->n_slots = atoi(mul);
+        while (isdigit(**psrc))
+            (*psrc)++;
+        hostdef->n_slots = val;
     }
     return 0;
 }
@@ -227,7 +237,6 @@ static int dcc_parse_ssh_host(struct dcc_hostdef *hostdef,
 static int dcc_parse_tcp_host(struct dcc_hostdef *hostdef,
                               const char * const token_start)
 {
-    char *port_str;
     int ret;
     const char *token = token_start;
     
@@ -245,22 +254,22 @@ static int dcc_parse_tcp_host(struct dcc_hostdef *hostdef,
 
     hostdef->port = dcc_default_port;
     if (token[0] == ':') {
+        char *tail;
+
         token++;
 
-        if ((ret = dcc_dup_part(&token, &port_str, " \t\n\f")) != 0)
-            return ret;
-        
-        if (port_str) {
-            char *tail;
-            hostdef->port = strtol(port_str, &tail, 10);
-            if (*tail != '\0' && !isspace(*tail)) {
-                rs_log_error("invalid tcp port specification in \"%s\"", port_str);
-                return EXIT_BAD_HOSTSPEC;
-            }
-            free(port_str);
+        hostdef->port = strtol(token, &tail, 10);
+        if (*tail != '\0' && !isspace(*tail) && *tail != '/') {
+            rs_log_error("invalid tcp port specification in \"%s\"", token);
+            return EXIT_BAD_HOSTSPEC; 
+        } else {
+            token = tail;
         }
     }
         
+    if ((ret = dcc_parse_multiplier(&token, hostdef)) != 0)
+        return ret;
+
     hostdef->mode = DCC_MODE_TCP;
     return 0;
 }
@@ -273,7 +282,15 @@ static int dcc_parse_localhost(struct dcc_hostdef *hostdef,
 
     hostdef->mode = DCC_MODE_LOCAL;
     hostdef->hostname = strdup("localhost");
-    hostdef->n_slots = 1;
+
+    /* Run only two tasks on localhost by default.
+     *
+     * It might be nice to run more if there are more CPUs, but determining
+     * the number of CPUs on Linux is a bit expensive since it requires
+     * examining mtab and /proc/stat.  Anyone lucky enough to have a >2 CPU
+     * machine can specify a number in the host list.
+     */
+    hostdef->n_slots = 2;
     
     return dcc_parse_multiplier(&token, hostdef);
 }
@@ -338,7 +355,8 @@ int dcc_parse_hosts(const char *where, const char *source_name,
             *ret_list = curr;   /* first */
         }
 
-        /* Default task limit */
+        /* Default task limit.  A bit higher than the local limit to allow for
+         * some files in transit. */
         curr->n_slots = 4;
             
         has_at = (memchr(token_start, '@', token_len) != NULL);
