@@ -61,7 +61,10 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <sys/select.h>
+#ifdef HAVE_SYS_SELECT_H
+#  include <sys/select.h>
+#endif
+#include <sys/time.h>
 
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -73,20 +76,49 @@
 #include "exitcode.h"
 
 
+
+const int timeout = 15;          /* seconds */
+
 int dcc_select_for_read(int fd)
 {
     fd_set fds;
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-    rs_trace("select for read on fd%d", fd);
-    if (select(fd + 1, &fds, NULL, NULL, NULL) == -1) {
-        rs_log_error("select failed: %s", strerror(errno));
-        return EXIT_IO_ERROR;
+    int rs;
+    struct timeval tv;
+
+    tv.tv_sec = timeout;
+    tv.tv_usec = 0;
+
+    while (1) {
+        FD_ZERO(&fds);
+        FD_SET(fd, &fds);
+
+        /* Linux updates the timeval to reflect the remaining time, but other
+         * OSs may not.  So on other systems, we may wait a bit too long if
+         * the client is interrupted -- but that won't happen very often so
+         * it's no big deal.
+         */
+
+        rs_trace("select for read on fd%d", fd);
+        rs = select(fd+1, NULL, &fds, NULL, &tv);
+        if (rs == -1 && errno == EINTR) {
+            continue;
+        } else if (rs == -1) {
+            rs_log_error("select() failed: %s", strerror(errno));
+            return EXIT_IO_ERROR;
+        } else if (rs == 0) {
+            rs_log_error("IO timeout");
+            return EXIT_IO_ERROR;
+        } else if (!FD_ISSET(fd, &fds)) {
+            rs_fatal("how did fd not get set?");
+        } else {
+            break;              /* woot */
+        }
     }
     return 0;
 }
 
 
+/* TODO: Handle EINTR */
 int dcc_select_for_write(int fd)
 {
     fd_set fds;
