@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/io.h,v 1.14 2002/08/31 04:13:56 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,19 +20,22 @@
  * USA
  */
 
-/* int dcc_dump_file(int ofd, const char *fname, const char *token); */
-
 int dcc_writex(int fd, const void *buf, size_t len);
-int dcc_write_str(int fd, const char *what);
-int dcc_write_int(int fd, unsigned v);
 
 int dcc_r_token(int ifd, char *token);
 
 int dcc_readx(int fd, void *buf, size_t len);
-int dcc_read_int(int fd, /*@out@*/ unsigned * v);
 int dcc_pump_readwrite(int ofd, int ifd, size_t n);
 int dcc_pump_sendfile(int ofd, int ifd, size_t n);
 int dcc_r_str_alloc(int fd, int len, char **buf);
 
 int tcp_cork_sock(int fd, int corked);
 int dcc_close(int fd);
+
+
+int dcc_select_for_write(int fd);
+int dcc_select_for_read(int fd);
+
+/* loadfile.c */
+int dcc_load_file(const char *filename,
+		  char **retbuf);
