@@ -148,7 +148,7 @@ static int sys_lock(int fd, int block)
 
 int dcc_unlock(int lock_fd)
 {
-    rs_trace("release lock");
+    rs_trace("release lock fd%d", lock_fd);
     /* All our current locks can just be closed */
     if (close(lock_fd)) {
         rs_log_error("close failed: %s", strerror(errno));
@@ -204,7 +204,8 @@ int dcc_lock_host(const char *lockname,
     }        
 
     if (sys_lock(*lock_fd, block) == 0) {
-        rs_trace("got %s lock on %s slot %d", lockname, host->hostdef_string, slot);
+        rs_trace("got %s lock on %s slot %d as fd%d", lockname,
+                 host->hostdef_string, slot, *lock_fd);
         free(fname);
         return 0;
     } else {
