@@ -101,7 +101,7 @@ int dcc_x_result_header(int ofd)
 }
 
 
-int dcc_x_cc_status(int ofd, int status)
+int dcc_x_cc_status(int ofd, unsigned status)
 {
     return dcc_x_token_int(ofd, "STAT", status);
 }
@@ -160,7 +160,7 @@ static int dcc_explain_mismatch(const char *buf,
  * @param expected 4-char token that is expected to come in next
  * @param val      receives the parameter value
  **/
-int dcc_r_token_int(int ifd, const char *expected, int *val)
+int dcc_r_token_int(int ifd, const char *expected, unsigned *val)
 {
     char buf[13], *bum;
     int ret;
@@ -220,3 +220,41 @@ int dcc_r_str_alloc(int fd, int l, char **buf)
 
      return 0;
 }
+
+
+/**
+ * Write a token, and then the string @p buf.
+ *
+ * The length of buf is determined by its nul delimiter, but the \0 is not sent.
+ **/
+int dcc_x_token_string(int fd,
+                       const char *token,
+                       const char *buf)
+{
+    int ret;
+    size_t len;
+
+    len = strlen(buf);
+    if ((ret = dcc_x_token_int(fd, token, (unsigned) len)))
+        return ret;
+    if ((ret = dcc_writex(fd, buf, len)))
+        return ret;
+
+    return 0;
+}
+
+
+int dcc_r_token_string(int ifd, const char *expect_token,
+                       char **p_str)
+{
+    unsigned a_len;
+    int ret;
+        
+    if ((ret = dcc_r_token_int(ifd, expect_token, &a_len)))
+        return ret;
+
+    if ((ret = dcc_r_str_alloc(ifd, a_len, p_str)))
+        return ret;
+    
+    return 0;
+}
