@@ -1,9 +1,8 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/rpc.h,v 1.12 2003/02/03 01:50:18 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,8 +26,19 @@
 int dcc_x_result_header(int ofd);
 int dcc_r_result_header(int ofd);
 
-int dcc_x_cc_status(int, int);
-int dcc_r_cc_status(int, int *);
+int dcc_x_cc_status(int, unsigned);
+int dcc_r_cc_status(int, unsigned *);
 
-int dcc_x_token_int(int ofd, const char *token, int param);
-int dcc_r_token_int(int ifd, const char *expected, int *val);
+int dcc_x_token_int(int ofd, const char *token, unsigned param);
+int dcc_r_token_int(int ifd, const char *expected, unsigned int *val);
+
+int dcc_x_token_string(int fd,
+                       const char *token,
+                       const char *buf);
+
+int dcc_r_token_string(int ifd, const char *expect_token,
+                       char **p_str);
+
+/* srvrpc.c */
+int dcc_r_request_header(int ifd);
+int dcc_r_argv(int ifd, /*@out@*/ char ***argv);
