@@ -82,70 +82,16 @@
 #include "srvnet.h"
 #include "filename.h"
 #include "hosts.h"
-
-
-static int dcc_r_request_header(int ifd)
-{
-    int vers;
-    int ret;
-    
-    if ((ret = dcc_r_token_int(ifd, "DIST", &vers)) != 0) {
-        rs_log_error("client did not provide distcc magic fairy dust");
-        return ret;
-    }
-
-    if (vers != PROTO_VER) {
-        rs_log_error("client version is %d, i am %d", vers, 1);
-        return EXIT_PROTOCOL_ERROR;
-    }
-    
-    return 0;
-}
+#include "daemon.h"
 
 
 /**
- * Read an argv[] vector from the network.
+ * We copy all serious distccd messages to this file, as well as sending the
+ * compiler errors there, so they're visible to the client.
  **/
-static int dcc_r_argv(int ifd, /*@out@*/ char ***argv)
-{
-    int i;
-    int argc;
-    char **a;
-    int ret;
-
-    *argv = NULL;
-     
-    if (dcc_r_token_int(ifd, "ARGC", &argc))
-        return EXIT_PROTOCOL_ERROR;
-
-    rs_trace("reading %d arguments from job submission", argc);
-    
-    /* Have to make the argv one element too long, so that it can be
-     * terminated by a null element. */
-    *argv = a = (char **) calloc((size_t) argc+1, sizeof a[0]);
-    if (a == NULL) {
-        rs_log_error("alloc failed");
-        return EXIT_OUT_OF_MEMORY;
-    }
-    a[argc] = NULL;
-
-    for (i = 0; i < argc; i++) {
-        int a_len;
-        
-        if ((ret = dcc_r_token_int(ifd, "ARGV", &a_len)))
-            return ret;
-
-        if ((ret = dcc_r_str_alloc(ifd, a_len, &a[i])))
-            return ret;
-        
-        rs_trace("argv[%d] = \"%s\"", i, a[i]);
-    }
-
-    dcc_trace_argv("got arguments", a);
-    
-    return 0;
-}
+static int dcc_compile_log_fd = -1;
 
+static int dcc_run_job(int netfd);
 
 
 /**
@@ -154,59 +100,43 @@ static int dcc_r_argv(int ifd, /*@out@*/ char ***argv)
  **/
 static int dcc_add_log_to_file(const char *err_fname)
 {
-    int log_fd;
-
-    log_fd = open(err_fname, O_WRONLY|O_CREAT|O_TRUNC, 0600);
-    if (log_fd == -1) {
+    if (dcc_compile_log_fd != -1) {
+        rs_fatal("compile log already open?");
+    }
+    
+    dcc_compile_log_fd = open(err_fname, O_WRONLY|O_CREAT|O_TRUNC, 0600);
+    if (dcc_compile_log_fd == -1) {
         rs_log_error("failed to open %s: %s", err_fname, strerror(errno));
         return EXIT_IO_ERROR;
     }
 
     /* Only send fairly serious errors back */
-    rs_add_logger(rs_logger_file, RS_LOG_WARNING, NULL, log_fd);
+    rs_add_logger(rs_logger_file, RS_LOG_WARNING, NULL, dcc_compile_log_fd);
+
+    return 0;
+}
+
+
+
+static int dcc_remove_log_to_file(void)
+{
+    if (dcc_compile_log_fd == -1) {
+        rs_fatal("compile log not open?");
+    }
+
+    /* must exactly match call in dcc_add_log_to_file */
+    rs_remove_logger(rs_logger_file, RS_LOG_WARNING, NULL,
+                     dcc_compile_log_fd);
+
+    dcc_close(dcc_compile_log_fd);
+
+    dcc_compile_log_fd = -1;
 
     return 0;
 }
 
 
 
-/**
- * On receipt of a fatal signal, kill the compiler as well.
- **/
-static void dcc_terminate_group(int whichsig)
-{
-    dcc_reset_signal(whichsig);
-    kill(0, whichsig);
-}
-
-
-
-/**
- * Signals are caught only in the parent at the moment, because we
- * want to be able to kill off all children and log a message when
- * exiting.  This is perhaps not the best thing to do -- it might be
- * better to simply let ourselves be terminated, and for children to
- * go away when they're finished their current work.  If the admin is
- * really impatient, they can kill all the children anyhow.
- **/
-static void dcc_hook_terminate_group(void)
-{
-    struct sigaction act_term;
-
-    memset(&act_term, 0, sizeof act_term);
-    act_term.sa_handler = dcc_terminate_group;
-
-    if (sigaction(SIGTERM, &act_term, NULL)
-        || sigaction(SIGHUP, &act_term, NULL)
-        || sigaction(SIGINT, &act_term, NULL)
-        || sigaction(SIGQUIT, &act_term, NULL)) {
-        rs_log_error("failed to install signal handlers: %s",
-                     strerror(errno));
-    }
-}
-
-
-              
 /**
  * Read and execute a job to/from socket @p ifd
  *
@@ -214,79 +144,81 @@ static void dcc_hook_terminate_group(void)
  *
  * @return standard exit code
  **/
-int dcc_accept_job(int netfd)
+int dcc_service_job(int netfd,
+                    struct sockaddr *cli_addr,
+                    int cli_len)
+{
+    int ret;
+
+    /* TODO: Don't check for ssh connections */
+    /* Log client name and check access if appropriate. */
+    if ((ret = dcc_check_client(cli_addr, cli_len)) != 0)
+        return ret;
+
+    return dcc_run_job(netfd);
+}
+
+
+static char *dcc_input_tmpnam(char * orig_input)
+{
+    const char *input_exten;
+    rs_trace("input file %s", orig_input);
+    input_exten = dcc_find_extension(orig_input);
+    if (input_exten)
+        input_exten = dcc_preproc_exten(input_exten);
+    if (!input_exten)           /* previous line might return NULL */
+        input_exten = ".tmp";
+    return dcc_make_tmpnam("server", input_exten);
+}
+
+
+static int dcc_run_job(int netfd)
 {
     char **argv;
-    int status;
+    unsigned status;
     char *temp_i, *temp_o, *err_fname, *out_fname;
     size_t o_size = 0;
     int ret, compile_ret;
     char *orig_input, *orig_output;
-    const char *input_exten;
-    long u_us, s_us;
-    int i_size;
+    unsigned i_size;
     pid_t cc_pid;
 
-    err_fname = dcc_make_tmpnam("cc", ".err");
-    out_fname = dcc_make_tmpnam("cc", ".out");
+    err_fname = dcc_make_tmpnam("cc", ".stderr");
+    out_fname = dcc_make_tmpnam("cc", ".stdout");
     dcc_remove_if_exists(err_fname);
     dcc_remove_if_exists(out_fname);
 
     /* Capture any messages relating to this compilation to the same file as
-     * compiler errors so that they can all be sent back to the client.
-     *
-     * XXX: This file is never removed from the logger, so we leak some fds
-     * here in --no-fork mode, and the logger list will keep getting longer.
-     * We'd need a rs_remove_logger, which would not be too hard. */
+     * compiler errors so that they can all be sent back to the client. */
     dcc_add_log_to_file(err_fname);
 
-    /* TODO: Don't call this for ssh connections */
-    
-    /* Log client name and check access if appropriate. */
-    if ((ret = dcc_check_client(netfd)) != 0)
-        return ret;
-
     /* Ignore SIGPIPE; we consistently check error codes and will see the
      * EPIPE.  Note that it is set back to the default behaviour when spawning
      * a child, to handle cases like the assembler dying while its being fed
      * from the compiler */
     dcc_ignore_sigpipe(1);
 
-    /* Enter a new process group, so that we can kill the compiler as well if
-     * we're terminated. */
-    dcc_setpgid(0, 0);
-    dcc_hook_terminate_group();
-
     /* Allow output to accumulate into big packets. */
     tcp_cork_sock(netfd, 1);
 
     if ((ret = dcc_r_request_header(netfd))
         || (ret = dcc_r_argv(netfd, &argv))
         || (ret = dcc_scan_args(argv, &orig_input, &orig_output, &argv)))
-        return ret;
+        goto out_cleanup;
     
-    /* TODO: Make sure cleanup is called in case of error.
-     *
-     * TODO: Move out the guts of this function. */
+    rs_trace("output file %s", orig_output);
 
-    rs_log_info("input file %s, output file %s", orig_input, orig_output);
-
-    input_exten = dcc_find_extension(orig_input);
-    if (input_exten)
-        input_exten = dcc_preproc_exten(input_exten);
-    if (!input_exten)           /* previous line might return NULL */
-        input_exten = ".tmp";
-    temp_i = dcc_make_tmpnam("server", input_exten);
-    temp_o = dcc_make_tmpnam("server", ".out");
+    temp_i = dcc_input_tmpnam(orig_input);
+    temp_o = dcc_make_tmpnam("server", ".o");
 
     if ((ret = dcc_r_token_int(netfd, "DOTI", &i_size))
         || (ret = dcc_r_file_timed(netfd, temp_i, (size_t) i_size))
         || (ret = dcc_set_input(argv, temp_i))
         || (ret = dcc_set_output(argv, temp_o)))
-        return ret;
+        goto out_cleanup;
 
     if ((compile_ret = dcc_spawn_child(argv, &cc_pid, "/dev/null", out_fname, err_fname))
-        || (compile_ret = dcc_collect_child(cc_pid, &status, &u_us, &s_us))) {
+        || (compile_ret = dcc_collect_child("cc", cc_pid, &status))) {
         /* We didn't get around to finding a wait status from the actual compiler */
         status = W_EXITCODE(compile_ret, 0);
     }
@@ -299,15 +231,16 @@ int dcc_accept_job(int netfd)
         || WEXITSTATUS(status)) {
         dcc_x_token_int(netfd, "DOTO", 0);
     } else {
-        ret = dcc_x_file_timed(netfd, temp_o, "DOTO", &o_size);
+        ret = dcc_x_file(netfd, temp_o, "DOTO", &o_size);
     }
 
-    dcc_report_rusage(argv[0], u_us, s_us);
     dcc_critique_status(status, argv[0], dcc_hostdef_local, 0);
     tcp_cork_sock(netfd, 0);
 
-    rs_log_info("complete; output file: %ld bytes", (long) o_size);
+    rs_log(RS_LOG_INFO|RS_LOG_NONAME, "job complete");
 
+    out_cleanup:
+    dcc_remove_log_to_file();
     dcc_cleanup_tempfiles();
 
     return ret;
