@@ -78,11 +78,12 @@
  **/
 static int dcc_listen_by_addr(int *listen_fd,
                               struct sockaddr *sa,
-                              size_t sa_len)
+                              size_t salen)
 {
     int one = 1;
     int fd;
-    
+    char *sa_buf = NULL;
+
     fd = socket(sa->sa_family, SOCK_STREAM, 0);
     if (fd == -1) {
 	rs_log_error("socket creation failed: %s", strerror(errno));
@@ -91,19 +92,26 @@ static int dcc_listen_by_addr(int *listen_fd,
 
     setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof(one));
 
+    dcc_sockaddr_to_string(sa, salen, &sa_buf);
+
     /* now we've got a socket - we need to bind it */
-    if (bind(fd, sa, sa_len) == -1) {
-        /* TODO: Print raw sockaddr form in some readable way. */
-	rs_log_error("bind failed: %s", strerror(errno));
+    if (bind(fd, sa, salen) == -1) {
+	rs_log_error("bind of %s failed: %s", sa_buf ? sa_buf : "UNKNOWN",
+                     strerror(errno));
+        free(sa_buf);
 	close(fd);
 	return EXIT_BIND_FAILED;
     }
 
+    rs_log_info("listening on %s", sa_buf ? sa_buf : "UNKNOWN");
+    free(sa_buf);
+
     if (listen(fd, 10)) {
         rs_log_error("listen failed: %s", strerror(errno));
+        close(fd);
         return EXIT_BIND_FAILED;
     }
-
+    
     *listen_fd = fd;
     return 0;
 
@@ -218,31 +226,17 @@ int is_a_socket(int fd)
 /**
  * Log client IP address and perform access control checks.
  **/
-int dcc_check_client(int fd)
+int dcc_check_client(struct sockaddr *psa,
+                     int salen)
 {
-#ifdef HAVE_SOCKADDR_STORAGE
-    struct sockaddr_storage ss;
-#else
-    struct sockaddr_in ss;
-#endif
-
-    struct sockaddr *psa = (struct sockaddr *) &ss;
-    
-    socklen_t len = sizeof ss;
     char *client_ip;
     struct dcc_allow_list *l;
     int ret;
-
-    if ((getpeername(fd, psa, &len) == -1)) {
-        rs_log_warning("failed to get peer name: %s", strerror(errno));
-        return 0;
-    }
-
-    if ((ret = dcc_sockaddr_to_ip(psa, len, &client_ip)) != 0)
+    
+    if ((ret = dcc_sockaddr_to_string(psa, salen, &client_ip)) != 0)
         return ret;
     
-    rs_log_info("connection from %s (address family %d)", client_ip,
-                psa->sa_family);
+    rs_log_info("connection from %s", client_ip);
     free(client_ip);
 
     /* if there are any access entries, then we must match one */
