@@ -1,7 +1,7 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/tempfile.h,v 1.1 2002/08/31 21:52:14 mbp Exp $ 
+ * $Header: /data/cvs/distcc/src/tempfile.h,v 1.3 2003/06/10 05:47:31 mbp Exp $ 
  *
  * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
  *
@@ -28,3 +28,6 @@ int dcc_setup_tempdir(void);
 void dcc_cleanup_tempfiles(void);
 char *dcc_make_tmpnam(const char *, const char *suffix);
 
+int dcc_enter_tempdir(void);
+
+int dcc_make_dir (const char *path);
