@@ -46,6 +46,7 @@
 #include "io.h"
 #include "util.h"
 #include "exitcode.h"
+#include "snprintf.h"
 
 
 			/* I will make a man more precious than fine
@@ -209,14 +210,8 @@ int set_cloexec_flag (int desc, int value)
  **/
 int dcc_ignore_sigpipe(int val)
 {
-    struct sigaction action;
-
-    action.sa_handler = val ? SIG_IGN : SIG_DFL;
-    sigemptyset(&action.sa_mask);
-    action.sa_flags = 0;
-
-    if (sigaction(SIGPIPE, &action, NULL)) {
-        rs_log_warning("sigaction(SIGPIPE, %s) failed: %s",
+    if (signal(SIGPIPE, val ? SIG_IGN : SIG_DFL) == SIG_ERR) {
+        rs_log_warning("signal(SIGPIPE, %s) failed: %s",
                        val ? "ignore" : "default",
                        strerror(errno));
         return EXIT_DISTCC_FAILED;
@@ -385,3 +380,16 @@ int dcc_dup_part(const char **psrc, char **pdst, const char *sep)
     return 0;
 }
 
+
+
+int dcc_remove_if_exists(const char *fname)
+{
+    if (unlink(fname) && errno != ENOENT) {
+        rs_log_warning("failed to unlink %s: %s", fname,
+                       strerror(errno));
+        return EXIT_IO_ERROR;
+    }
+    return 0;
+}
+
+
