@@ -152,6 +152,8 @@ Copyright (C) 2002, 2003 by Martin Pool
 
 # TODO: Test backoff from downed hosts.
 
+# TODO: Check again in --no-prefork mode.
+
 
 import time, sys, string, os, types, re, popen2, pprint
 import signal, os.path, string
@@ -213,6 +215,7 @@ as soon as that happens we can go ahead and start the client."""
 
     def killDaemon(self):
         import signal, time
+
         try:
             pid = int(open(self.daemon_pidfile, 'rt').read())
         except IOError:
@@ -232,7 +235,7 @@ as soon as that happens we can go ahead and start the client."""
 
     def daemon_command(self):
         """Return command to start the daemon"""
-        return ("distccd --verbose --lifetime=%d --daemon --log-file %s "
+        return ("distccd  --verbose --lifetime=%d --daemon --log-file %s "
                 "--pid-file %s --port %d --allow 127.0.0.1"
                 % (self.daemon_lifetime(),
                    self.daemon_logfile, self.daemon_pidfile, self.server_port))
@@ -452,15 +455,22 @@ class ParseHostSpec_Case(SimpleDistCC_Case):
         spec="""localhost 127.0.0.1 @angry   ted@angry
         \t@angry:/home/mbp/bin/distccd  angry:4204
         ipv4-localhost
+        angry/44
+        angry:300/44
+        angry/44:300
         """
-        expected="""7
-   1 LOCAL
+
+        expected="""10
+   2 LOCAL
    4 TCP 127.0.0.1 3632
    4 SSH (no-user) angry (no-command)
    4 SSH ted angry (no-command)
    4 SSH (no-user) angry /home/mbp/bin/distccd
    4 TCP angry 4204
    4 TCP ipv4-localhost 3632
+  44 TCP angry 3632
+  44 TCP angry 300
+  44 TCP angry 300
 """
         out, err = self.runcmd("DISTCC_HOSTS=\"%s\" h_hosts" % spec)
         assert out == expected, "expected %s\ngot %s" % (`expected`, `out`)
@@ -633,8 +643,6 @@ class NoDetachDaemon_Case(CompileHello_Case):
         os.kill(self.pid, signal.SIGTERM)
         pid, ret = os.wait()
         self.assert_equal(self.pid, pid)
-        self.assert_equal(ret, 0)
-
         
 
 class ImplicitCompiler_Case(CompileHello_Case):
@@ -1024,9 +1032,10 @@ class ParseMask_Case(comfychair.TestCase):
         ]
     def runtest(self):
         for mask, client, expected in ParseMask_Case.values:
-            ret, msgs, err = self.runcmd_unchecked("h_parsemask %s %s" % (mask, client))
+            cmd = "h_parsemask %s %s" % (mask, client)
+            ret, msgs, err = self.runcmd_unchecked(cmd)
             if ret != expected:
-                self.fail("%s %s -> %d" % (mask, client, ret))
+                self.fail("%s gave %d, expected %d" % (cmd, ret, expected))
 
 
 class HostFile_Case(CompileHello_Case):
